# -*- coding: utf-8 -*-
"""
The Degreed Integrated Channel package.
"""

__version__ = "0.1.0"

default_app_config = ("integrated_channels.degreed.apps."  # pylint: disable=invalid-name
                      "DegreedConfig")
