# Generated by Django 2.2.16 on 2020-09-30 17:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('blackboard', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='blackboardenterprisecustomerconfiguration',
            name='client_id',
            field=models.CharField(help_text='The API Client ID provided to edX by the enterprise customer to be used to make API calls on behalf of the customer. Called Application Key in Blackboard', max_length=255, null=True, verbose_name='API Client ID or Blackboard Application Key'),
        ),
        migrations.AlterField(
            model_name='blackboardenterprisecustomerconfiguration',
            name='client_secret',
            field=models.CharField(help_text='The API Client Secret provided to edX by the enterprise customer to be used to make  API calls on behalf of the customer. Called Application Secret in Blackboard', max_length=255, null=True, verbose_name='API Client Secret or Application Secret'),
        ),
        migrations.AlterField(
            model_name='historicalblackboardenterprisecustomerconfiguration',
            name='client_id',
            field=models.CharField(help_text='The API Client ID provided to edX by the enterprise customer to be used to make API calls on behalf of the customer. Called Application Key in Blackboard', max_length=255, null=True, verbose_name='API Client ID or Blackboard Application Key'),
        ),
        migrations.AlterField(
            model_name='historicalblackboardenterprisecustomerconfiguration',
            name='client_secret',
            field=models.CharField(help_text='The API Client Secret provided to edX by the enterprise customer to be used to make  API calls on behalf of the customer. Called Application Secret in Blackboard', max_length=255, null=True, verbose_name='API Client Secret or Application Secret'),
        ),
    ]
